"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
const algorithm_1 = require("@phosphor/algorithm");
const widgets_1 = require("@phosphor/widgets");
const apputils_1 = require("@jupyterlab/apputils");
const coreutils_1 = require("@jupyterlab/coreutils");
const observables_1 = require("@jupyterlab/observables");
const contents_1 = require("./contents");
/**
 * The base url for a mybinder deployment.
 */
const MY_BINDER_BASE_URL = 'https://mybinder.org/v2/gh';
/**
 * The GitHub base url.
 */
exports.DEFAULT_GITHUB_BASE_URL = 'https://github.com';
/**
 * The className for disabling the mybinder button.
 */
const MY_BINDER_DISABLED = 'jp-MyBinderButton-disabled';
/**
 * Widget for hosting the GitHub filebrowser.
 */
class GitHubFileBrowser extends widgets_1.Widget {
    constructor(browser, drive) {
        super();
        this._binderActive = false;
        this._changeGuard = false;
        this.addClass('jp-GitHubBrowser');
        this.layout = new widgets_1.PanelLayout();
        this.layout.addWidget(browser);
        this._browser = browser;
        this._drive = drive;
        // Create an editable name for the user/org name.
        this.userName = new GitHubEditableName('', '<Edit User>');
        this.userName.addClass('jp-GitHubEditableUserName');
        this.userName.node.title = 'Click to edit user/organization';
        this._browser.toolbar.addItem('user', this.userName);
        this.userName.name.changed.connect(this._onUserChanged, this);
        this.baseUrl = exports.DEFAULT_GITHUB_BASE_URL;
        // Create a button that opens GitHub at the appropriate
        // repo+directory.
        this._openGitHubButton = new apputils_1.ToolbarButton({
            onClick: () => {
                let url = this.baseUrl;
                // If there is no valid user, open the GitHub homepage.
                if (!this._drive.validUser) {
                    window.open(url);
                    return;
                }
                const localPath = this._browser.model.manager.services.contents.localPath(this._browser.model.path);
                const resource = contents_1.parsePath(localPath);
                url = coreutils_1.URLExt.join(url, resource.user);
                if (resource.repository) {
                    url = coreutils_1.URLExt.join(url, resource.repository, 'tree', 'master', resource.path);
                }
                window.open(url);
            },
            className: 'jp-GitHubIcon',
            tooltip: 'Open this repository on GitHub'
        });
        this._browser.toolbar.addItem('GitHub', this._openGitHubButton);
        // Create a button the opens MyBinder to the appropriate repo.
        this._launchBinderButton = new apputils_1.ToolbarButton({
            onClick: () => {
                // If binder is not active for this directory, do nothing.
                if (!this._binderActive) {
                    return;
                }
                const localPath = this._browser.model.manager.services.contents.localPath(this._browser.model.path);
                const resource = contents_1.parsePath(localPath);
                const url = coreutils_1.URLExt.join(MY_BINDER_BASE_URL, resource.user, resource.repository, 'master');
                // Attempt to open using the JupyterLab tree handler
                const tree = coreutils_1.URLExt.join('lab', 'tree', resource.path);
                window.open(url + `?urlpath=${tree}`);
            },
            tooltip: 'Launch this repository on mybinder.org',
            className: 'jp-MyBinderButton'
        });
        this._browser.toolbar.addItem('binder', this._launchBinderButton);
        // Set up a listener to check if we can launch mybinder.
        this._browser.model.pathChanged.connect(this._onPathChanged, this);
        // Trigger an initial pathChanged to check for binder state.
        this._onPathChanged();
        this._drive.rateLimitedState.changed.connect(this._updateErrorPanel, this);
    }
    /**
     * The GitHub base URL
     */
    get baseUrl() {
        return this._baseUrl;
    }
    /**
     * The GitHub base URL is set by the settingsRegistry change hook
     */
    set baseUrl(url) {
        this._baseUrl = url;
    }
    /**
     * React to a change in user.
     */
    _onUserChanged(sender, args) {
        if (this._changeGuard) {
            return;
        }
        this._changeGuard = true;
        this._browser.model.cd(`/${args.newValue}`).then(() => {
            this._changeGuard = false;
            this._updateErrorPanel();
            // Once we have the new listing, maybe give the file listing
            // focus. Once the input element is removed, the active element
            // appears to revert to document.body. If the user has subsequently
            // focused another element, don't focus the browser listing.
            if (document.activeElement === document.body) {
                const listing = this._browser.layout.widgets[2];
                listing.node.focus();
            }
        });
    }
    /**
     * React to the path changing for the browser.
     */
    _onPathChanged() {
        const localPath = this._browser.model.manager.services.contents.localPath(this._browser.model.path);
        const resource = contents_1.parsePath(localPath);
        // If we are not already changing the user name, set it.
        if (!this._changeGuard) {
            this._changeGuard = true;
            this.userName.name.set(resource.user);
            this._changeGuard = false;
            this._updateErrorPanel();
        }
        // Check for a valid user.
        if (!this._drive.validUser) {
            this._launchBinderButton.addClass(MY_BINDER_DISABLED);
            this._binderActive = false;
            return;
        }
        // Check for a valid repo.
        if (!resource.repository) {
            this._launchBinderButton.addClass(MY_BINDER_DISABLED);
            this._binderActive = false;
            return;
        }
        // If we are in the root of the repository, check for one of
        // the special files indicating we can launch the repository on mybinder.
        // TODO: If the user navigates to a subdirectory without hitting the root
        // of the repository, we will not check for whether the repo is binder-able.
        // Figure out some way around this.
        if (resource.path === '') {
            const item = algorithm_1.find(this._browser.model.items(), i => {
                return (i.name === 'requirements.txt' ||
                    i.name === 'environment.yml' ||
                    i.name === 'apt.txt' ||
                    i.name === 'REQUIRE' ||
                    i.name === 'Dockerfile' ||
                    (i.name === 'binder' && i.type === 'directory'));
            });
            if (item) {
                this._launchBinderButton.removeClass(MY_BINDER_DISABLED);
                this._binderActive = true;
                return;
            }
            else {
                this._launchBinderButton.addClass(MY_BINDER_DISABLED);
                this._binderActive = false;
                return;
            }
        }
        // If we got this far, we are in a subdirectory of a valid
        // repository, and should not change the binderActive status.
        return;
    }
    /**
     * React to a change in the validity of the drive.
     */
    _updateErrorPanel() {
        const localPath = this._browser.model.manager.services.contents.localPath(this._browser.model.path);
        const resource = contents_1.parsePath(localPath);
        const rateLimited = this._drive.rateLimitedState.get();
        const validUser = this._drive.validUser;
        // If we currently have an error panel, remove it.
        if (this._errorPanel) {
            const listing = this._browser.layout.widgets[2];
            listing.node.removeChild(this._errorPanel.node);
            this._errorPanel.dispose();
            this._errorPanel = null;
        }
        // If we are being rate limited, make an error panel.
        if (rateLimited) {
            this._errorPanel = new GitHubErrorPanel('You have been rate limited by GitHub! ' +
                'You will need to wait about an hour before ' +
                'continuing');
            const listing = this._browser.layout.widgets[2];
            listing.node.appendChild(this._errorPanel.node);
            return;
        }
        // If we have an invalid user, make an error panel.
        if (!validUser) {
            const message = resource.user
                ? `"${resource.user}" appears to be an invalid user name!`
                : 'Please enter a GitHub user name';
            this._errorPanel = new GitHubErrorPanel(message);
            const listing = this._browser.layout.widgets[2];
            listing.node.appendChild(this._errorPanel.node);
            return;
        }
    }
}
exports.GitHubFileBrowser = GitHubFileBrowser;
/**
 * A widget that hosts an editable field,
 * used to host the currently active GitHub
 * user name.
 */
class GitHubEditableName extends widgets_1.Widget {
    constructor(initialName = '', placeholder) {
        super();
        this._pending = false;
        this.addClass('jp-GitHubEditableName');
        this._nameNode = document.createElement('div');
        this._nameNode.className = 'jp-GitHubEditableName-display';
        this._editNode = document.createElement('input');
        this._editNode.className = 'jp-GitHubEditableName-input';
        this._placeholder = placeholder || '<Edit Name>';
        this.node.appendChild(this._nameNode);
        this.name = new observables_1.ObservableValue(initialName);
        this._nameNode.textContent = initialName || this._placeholder;
        this.node.onclick = () => {
            if (this._pending) {
                return;
            }
            this._pending = true;
            Private.changeField(this._nameNode, this._editNode).then(value => {
                this._pending = false;
                if (this.name.get() !== value) {
                    this.name.set(value);
                }
            });
        };
        this.name.changed.connect((s, args) => {
            if (args.oldValue !== args.newValue) {
                this._nameNode.textContent =
                    args.newValue || this._placeholder;
            }
        });
    }
}
exports.GitHubEditableName = GitHubEditableName;
/**
 * A widget hosting an error panel for the browser,
 * used if there is an invalid user name or if we
 * are being rate-limited.
 */
class GitHubErrorPanel extends widgets_1.Widget {
    constructor(message) {
        super();
        this.addClass('jp-GitHubErrorPanel');
        const image = document.createElement('div');
        const text = document.createElement('div');
        image.className = 'jp-GitHubErrorImage';
        text.className = 'jp-GitHubErrorText';
        text.textContent = message;
        this.node.appendChild(image);
        this.node.appendChild(text);
    }
}
exports.GitHubErrorPanel = GitHubErrorPanel;
/**
 * A module-Private namespace.
 */
var Private;
(function (Private) {
    function changeField(text, edit) {
        // Replace the text node with an the input element.
        let parent = text.parentElement;
        let initialValue = text.textContent || '';
        edit.value = initialValue;
        parent.replaceChild(edit, text);
        edit.focus();
        // Highlight the input element
        let index = edit.value.lastIndexOf('.');
        if (index === -1) {
            edit.setSelectionRange(0, edit.value.length);
        }
        else {
            edit.setSelectionRange(0, index);
        }
        return new Promise((resolve, reject) => {
            edit.onblur = () => {
                // Set the text content of the original node, then
                // replace the node.
                parent.replaceChild(text, edit);
                text.textContent = edit.value || initialValue;
                resolve(edit.value);
            };
            edit.onkeydown = (event) => {
                switch (event.keyCode) {
                    case 13: // Enter
                        event.stopPropagation();
                        event.preventDefault();
                        edit.blur();
                        break;
                    case 27: // Escape
                        event.stopPropagation();
                        event.preventDefault();
                        edit.value = initialValue;
                        edit.blur();
                        break;
                    default:
                        break;
                }
            };
        });
    }
    Private.changeField = changeField;
})(Private || (Private = {}));
